#!/usr/bin/env node
'use strict';

var React = require('react');
var ink = require('ink');
var yargs = require('yargs');
var createStoreHook = require('zustand');
var fs = require('fs');
var util = require('util');
var path = require('path');
var nanoid = require('nanoid');
var create = require('zustand/vanilla');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var yargs__default = /*#__PURE__*/_interopDefaultLegacy(yargs);
var createStoreHook__default = /*#__PURE__*/_interopDefaultLegacy(createStoreHook);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var create__default = /*#__PURE__*/_interopDefaultLegacy(create);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var MODE;

(function (MODE) {
  MODE["NORMAL"] = "NORMAL_MODE";
  MODE["INSERT"] = "INSERT_MODE";
})(MODE || (MODE = {}));

var TASK_TYPE;

(function (TASK_TYPE) {
  TASK_TYPE["DRAFT"] = "DRAFT_TASK";
  TASK_TYPE["ONGOING"] = "ONGOING_TASK";
  TASK_TYPE["DRAFT_ONGOING"] = "DRAFT_ONGOING_TASK";
  TASK_TYPE["FINISHED"] = "FINISHED_TASK";
})(TASK_TYPE || (TASK_TYPE = {}));

/* eslint-disable @typescript-eslint/no-explicit-any */

/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
function insertAtIndex(array, index, item, insertExactlyAtIndex) {
  var copy = array.slice();
  copy.splice(index, insertExactlyAtIndex ? 1 : 0, item);
  return copy;
}
function removeAtIndex(array, index) {
  var copy = array.slice();
  copy.splice(index, 1);
  return copy;
}
/* eslint-enable @typescript-eslint/no-explicit-any */

/* eslint-enable @typescript-eslint/explicit-module-boundary-types */

function createTask(taskType, taskName) {
  if (taskName === void 0) {
    taskName = "";
  }

  return {
    id: nanoid.nanoid(),
    name: taskName,
    type: taskType
  };
}
var DEFAULT_TASK_MACHINE_STATE = {
  origDraftOngoingTaskName: null,
  tasks: [],
  currentRow: -1,
  currentCol: -1,
  mode: MODE.NORMAL
};
var taskMachine = create__default['default'](function (set, get) {
  return __assign(__assign({}, DEFAULT_TASK_MACHINE_STATE), {
    getCurrentTask: function () {
      var _a = get(),
          tasks = _a.tasks,
          currentRow = _a.currentRow;

      return tasks[currentRow];
    },
    transition: function (input, key) {
      switch (get().mode) {
        case MODE.NORMAL:
          {
            switch (true) {
              case input === "j":
                set(function (state) {
                  return {
                    currentRow: (state.currentRow + 1) % state.tasks.length
                  };
                });
                break;

              case input === "k":
                set(function (state) {
                  var prevRow = state.currentRow - 1;
                  return {
                    currentRow: prevRow < 0 ? state.tasks.length - 1 : prevRow
                  };
                });
                break;

              case input === "i":
                set(function (state) {
                  return {
                    mode: MODE.INSERT,
                    currentRow: state.currentRow + 1,
                    currentCol: 0,
                    tasks: insertAtIndex(state.tasks, state.currentRow + 1, createTask(TASK_TYPE.DRAFT))
                  };
                });
                break;

              case input === "u":
                set(function (state) {
                  var currentTask = state.getCurrentTask();

                  if (state.tasks.length === 0 || currentTask.type === TASK_TYPE.FINISHED) {
                    return state;
                  }

                  var newTasks = insertAtIndex(state.tasks, state.currentRow, __assign(__assign({}, currentTask), {
                    type: TASK_TYPE.DRAFT_ONGOING
                  }), true);
                  return {
                    mode: MODE.INSERT,
                    tasks: newTasks,
                    origDraftOngoingTaskName: currentTask.name,
                    currentCol: currentTask.name.length
                  };
                });
                break;

              case input === "D":
                set(function (state) {
                  if (state.tasks.length === 0) {
                    return state;
                  }

                  return {
                    tasks: removeAtIndex(state.tasks, state.currentRow),
                    currentRow: state.currentRow === state.tasks.length - 1 ? state.currentRow - 1 : state.currentRow
                  };
                });
                break;

              case input === " ":
                set(function (state) {
                  var currentTask = state.getCurrentTask();
                  return {
                    tasks: insertAtIndex(state.tasks, state.currentRow, __assign(__assign({}, currentTask), {
                      type: currentTask.type === TASK_TYPE.ONGOING ? TASK_TYPE.FINISHED : TASK_TYPE.ONGOING
                    }), true)
                  };
                });
                break;
            }

            break;
          }

        case MODE.INSERT:
          {
            switch (true) {
              case key === null || key === void 0 ? void 0 : key.escape:
                set(function (state) {
                  var currentTask = state.getCurrentTask();
                  var nextCurrentRow;
                  var nextTasks;

                  switch (currentTask.type) {
                    case TASK_TYPE.DRAFT:
                      nextCurrentRow = state.currentRow - 1;
                      nextTasks = removeAtIndex(state.tasks, state.currentRow);
                      break;

                    case TASK_TYPE.DRAFT_ONGOING:
                      nextCurrentRow = state.currentRow;
                      nextTasks = insertAtIndex(state.tasks, state.currentRow, __assign(__assign({}, currentTask), {
                        type: TASK_TYPE.ONGOING,
                        name: state.origDraftOngoingTaskName
                      }), true);
                      break;
                  }

                  return {
                    origDraftOngoingTaskName: DEFAULT_TASK_MACHINE_STATE.origDraftOngoingTaskName,
                    mode: MODE.NORMAL,
                    currentRow: nextCurrentRow,
                    tasks: nextTasks
                  };
                });
                break;

              case (key === null || key === void 0 ? void 0 : key.ctrl) && input === "o":
                set(function (state) {
                  var draftTask = state.tasks[state.currentRow];

                  if (!draftTask.name) {
                    return state;
                  }

                  var newTasks = insertAtIndex(state.tasks, state.currentRow, createTask(TASK_TYPE.ONGOING, draftTask.name));
                  var nextCurrentRow = state.currentRow + 1;
                  newTasks[nextCurrentRow].name = "";
                  return {
                    tasks: newTasks,
                    currentCol: 0,
                    currentRow: nextCurrentRow
                  };
                });
                break;

              case (key === null || key === void 0 ? void 0 : key.ctrl) && input === "a":
                set(function () {
                  return {
                    currentCol: 0
                  };
                });
                break;

              case (key === null || key === void 0 ? void 0 : key.ctrl) && input === "e":
                set(function (state) {
                  return {
                    currentCol: state.getCurrentTask().name.length
                  };
                });
                break;

              case (key === null || key === void 0 ? void 0 : key.ctrl) && input === "u":
                set(function (state) {
                  return {
                    currentCol: 0,
                    tasks: insertAtIndex(state.tasks, state.currentRow, __assign(__assign({}, state.getCurrentTask()), {
                      name: ""
                    }), true)
                  };
                });
                break;

              case key === null || key === void 0 ? void 0 : key["return"]:
                set(function (state) {
                  var draftTask = state.tasks[state.currentRow];

                  if (!draftTask.name) {
                    return state;
                  }

                  var newTasks = insertAtIndex(state.tasks, state.currentRow, createTask(TASK_TYPE.ONGOING, draftTask.name), true);
                  return {
                    mode: MODE.NORMAL,
                    tasks: newTasks
                  };
                });
                break;

              case key === null || key === void 0 ? void 0 : key.leftArrow:
                set(function (state) {
                  return {
                    currentCol: Math.max(state.currentCol - 1, 0)
                  };
                });
                break;

              case key === null || key === void 0 ? void 0 : key.rightArrow:
                set(function (state) {
                  return {
                    currentCol: Math.min(state.currentCol + 1, state.getCurrentTask().name.length)
                  };
                });
                break;

              case key === null || key === void 0 ? void 0 : key["delete"]:
                set(function (state) {
                  if (state.currentCol === 0) {
                    return state;
                  }

                  var currentTask = state.getCurrentTask();
                  var nextCurrentCol = Math.max(state.currentCol - 1, 0);
                  var nextDraftTaskName = removeAtIndex(currentTask.name.split(""), nextCurrentCol).join("");
                  var nexTasks = insertAtIndex(state.tasks, state.currentRow, __assign(__assign({}, currentTask), {
                    name: nextDraftTaskName
                  }), true);
                  return {
                    tasks: nexTasks,
                    currentCol: nextCurrentCol
                  };
                });
                break;

              default:
                set(function (state) {
                  var draftTask = state.getCurrentTask();
                  var newDraftTaskName = insertAtIndex(draftTask.name.split(""), state.currentCol, input).join("");
                  var newTasks = insertAtIndex(state.tasks, state.currentRow, __assign(__assign({}, draftTask), {
                    name: newDraftTaskName
                  }), true);
                  return {
                    tasks: newTasks,
                    currentCol: state.currentCol + input.length
                  };
                });
                break;
            }

            break;
          }
      }
    }
  });
});

var readFile = util.promisify(fs.readFile);
var writeFile = util.promisify(fs.writeFile);
var useTaskMachine = createStoreHook__default['default'](taskMachine);
function useBootFromSaveFile(saveFile) {
  React.useEffect(function () {
    if (saveFile && path__default['default'].extname(saveFile) === ".json") {
      readFile(saveFile, "utf8").then(function (jsonFile) {
        var initialState = JSON.parse(jsonFile);
        useTaskMachine.setState(initialState);
      })["catch"](function () {
        return void 0;
      }); // DO NOTHING
    }
  }, [saveFile]);
}
function useWriteToSaveFile(saveFile) {
  React.useEffect(function () {
    return useTaskMachine.subscribe(function (state) {
      if (saveFile && path__default['default'].extname(saveFile) === ".json") {
        var saveState = JSON.stringify(state);
        writeFile(saveFile, saveState);
      }
    });
  }, [saveFile]);
}

function TaskItem(props) {
  var task = props.task,
      currentCol = props.currentCol,
      isFocused = props.isFocused;
  var taskName = task.name;

  if (task.type === TASK_TYPE.DRAFT || task.type === TASK_TYPE.DRAFT_ONGOING) {
    taskName = (task.name + " ").split("").map(function (c, i) {
      return /*#__PURE__*/React__default['default'].createElement(ink.Text, {
        key: i,
        inverse: i === currentCol
      }, c);
    });
  }

  var taskTypeIndicator;

  switch (task.type) {
    case TASK_TYPE.DRAFT:
    case TASK_TYPE.DRAFT_ONGOING:
      taskTypeIndicator = " * ";
      break;

    case TASK_TYPE.ONGOING:
      taskTypeIndicator = "[ ]";
      break;

    case TASK_TYPE.FINISHED:
      taskTypeIndicator = "[x]";
      break;
  }

  return /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    marginBottom: 1
  }, /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    alignItems: "flex-start"
  }, /*#__PURE__*/React__default['default'].createElement(ink.Text, {
    color: "cyanBright",
    bold: true
  }, isFocused ? "\u279c" : " ")), /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    marginX: 1,
    alignItems: "flex-start"
  }, /*#__PURE__*/React__default['default'].createElement(ink.Text, {
    dimColor: task.type === TASK_TYPE.FINISHED
  }, taskTypeIndicator)), /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    flexGrow: 1
  }, /*#__PURE__*/React__default['default'].createElement(ink.Text, {
    dimColor: task.type === TASK_TYPE.FINISHED,
    strikethrough: task.type === TASK_TYPE.FINISHED
  }, taskName)));
}

function Tasks(props) {
  var filePath = props.filePath;
  var taskMachine = useTaskMachine();
  var tasks = taskMachine.tasks,
      transition = taskMachine.transition,
      currentRow = taskMachine.currentRow,
      currentCol = taskMachine.currentCol;
  useBootFromSaveFile(filePath);
  useWriteToSaveFile(filePath);
  ink.useInput(transition);

  if (tasks.length === 0) {
    return /*#__PURE__*/React__default['default'].createElement(ink.Text, {
      color: "red"
    }, "No tasks created yet");
  }

  return /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    flexDirection: "column"
  }, tasks.map(function (task, index) {
    return /*#__PURE__*/React__default['default'].createElement(TaskItem, {
      key: task.id,
      task: task,
      isFocused: currentRow === index,
      currentCol: currentCol
    });
  }));
}

function App(props) {
  var title = props.title,
      filePath = props.filePath;
  return /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    padding: 1
  }, /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    padding: 1,
    borderStyle: "bold",
    flexDirection: "column",
    width: 56
  }, /*#__PURE__*/React__default['default'].createElement(ink.Box, {
    marginBottom: 1
  }, /*#__PURE__*/React__default['default'].createElement(ink.Text, {
    color: "magenta"
  }, title)), /*#__PURE__*/React__default['default'].createElement(Tasks, {
    filePath: filePath
  })));
}

var argv = yargs__default['default'](process.argv.slice(2)).scriptName("ishould").command("$0 [title] [options]", "NodeJS interactive CLI TODO list", function (yargs) {
  yargs.positional("title", {
    "default": "TODO",
    type: "string",
    describe: "The title of the TODO list"
  });
  yargs.positional("file", {
    type: "string",
    describe: "File to boot from and save into",
    alias: "f"
  });
}).alias("v", "version").alias("h", "help").help().argv;
ink.render( /*#__PURE__*/React__default['default'].createElement(App, {
  title: argv.title,
  filePath: argv.file
}));
